from .sessions import Session
from .__version__ import __title__


def request(method, url, **kwargs):
    with Session() as session:
        return session.request(method=method, url=url, **kwargs)


def raw(url, data, **kwargs):
    kwargs.setdefault('allow_redirects', True)
    return request(__title__, url, data=data, **kwargs)



__title__ = 'requests-raw'
__description__ = 'Use requests to talk HTTP via a Raw sockets (To Test RFC Compliance)'
__url__ = 'https://github.com/realgam3/requests-raw'
__version__ = '1.0.0'
__build__ = 0x010000
__author__ = 'Tomer Zait (realgam3)'
__author_email__ = 'realgam3@gmail.com'
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2020 Tomer Zait'



from .__version__ import __title__
from requests.utils import urlparse
from urllib3.connection import HTTPConnection, HTTPSConnection

try:
    from http import client
except ImportError:
    import httplib as client


class RawHTTPConnection(HTTPConnection):
    def __init__(self, *args, **kwargs):
        super(RawHTTPConnection, self).__init__(*args, **kwargs)
        self.__url = None
        self.__method = None

    def putrequest(self, method, url, *args, **kwargs):
        self.__method = method.lower()
        if self.__method != __title__:
            return super(RawHTTPConnection, self).putrequest(
                method, url, *args, **kwargs
            )

        self.__url = urlparse(url)
        if self._HTTPConnection__response and self._HTTPConnection__response.isclosed():
            self._HTTPConnection__response = None

        if self._HTTPConnection__state == client._CS_IDLE:
            self._HTTPConnection__state = client._CS_REQ_STARTED
        else:
            raise client.CannotSendRequest(self._HTTPConnection__state)

    def putheader(self, header, *values):
        if self.__method != __title__:
            return super(RawHTTPConnection, self).putheader(header, *values)

        if self._HTTPConnection__state != client._CS_REQ_STARTED:
            raise client.CannotSendHeader()

    def endheaders(self, message_body=None, **kwargs):
        if self.__method != __title__:
            return super(RawHTTPConnection, self).endheaders(message_body=message_body, **kwargs)

        buffer = message_body
        # HTTP Proxy
        if self.__url.scheme and self.__url.netloc:
            _buffer = buffer.split(b"/", 1)
            if _buffer[0].endswith(b" "):
                _buffer.insert(1, "{url.scheme}://{url.netloc}/".format(url=self.__url).encode())
            else:
                _buffer.insert(1, b"/")
            buffer = b"".join(_buffer)
        if self._HTTPConnection__state == client._CS_REQ_STARTED:
            self._HTTPConnection__state = client._CS_REQ_SENT
        else:
            raise client.CannotSendHeader()
        self.send(buffer)


class RawHTTPSConnection(HTTPSConnection):
    def __init__(self, *args, **kwargs):
        super(RawHTTPSConnection, self).__init__(*args, **kwargs)

    def putrequest(self, method, url, *args, **kwargs):
        self.__method = method.lower()
        if self.__method != __title__:
            return super(RawHTTPSConnection, self).putrequest(
                method, url, *args, **kwargs
            )

        if self._HTTPConnection__response and self._HTTPConnection__response.isclosed():
            self._HTTPConnection__response = None

        if self._HTTPConnection__state == client._CS_IDLE:
            self._HTTPConnection__state = client._CS_REQ_STARTED
        else:
            raise client.CannotSendRequest(self._HTTPConnection__state)

    def putheader(self, header, *values):
        if self.__method != __title__:
            return super(RawHTTPSConnection, self).putheader(header, *values)

        if self._HTTPConnection__state != client._CS_REQ_STARTED:
            raise client.CannotSendHeader()

    def endheaders(self, message_body=None, **kwargs):
        if self.__method != __title__:
            return super(RawHTTPSConnection, self).endheaders(message_body=message_body, **kwargs)

        if self._HTTPConnection__state == client._CS_REQ_STARTED:
            self._HTTPConnection__state = client._CS_REQ_SENT
        else:
            raise client.CannotSendHeader()
        self.send(message_body)



import requests
from requests import *
from .api import request, raw
from .adapters import RawAdapter
from .sessions import Session, session
from .__version__ import (__version__, __build__,
                          __title__, __description__,
                          __author__, __author_email__,
                          __url__, __license__, __copyright__)

# Original session request function
_request = requests.Session.request


# Fixes Bug https://github.com/realgam3/requests-raw/issues/1
def __is_response_to_head(response):
    method = response._method or __title__
    if isinstance(method, int):
        return method == 3
    return method.upper() == "HEAD"


# In case there is a Session object made before importing requests_raw
def __request(self, method, url, *args, **kwargs):
    if not getattr(self, "_raw_adapter_mounted", False):
        self.mount("http://", RawAdapter())
        self.mount("https://", RawAdapter())
        setattr(self, "_raw_adapter_mounted", True)
    return _request(self, method, url, *args, **kwargs)


def monkey_patch_all():
    setattr(requests, "raw", raw)
    setattr(requests.api, "raw", raw)
    setattr(requests.sessions.Session, "raw", Session.raw)
    setattr(requests.sessions.Session, "request", __request)
    setattr(requests.packages.urllib3.response, "is_response_to_head", __is_response_to_head)


# Monkey Patch Automatically
monkey_patch_all()



from .connection import RawHTTPConnection, RawHTTPSConnection
from urllib3.connectionpool import HTTPConnectionPool, HTTPSConnectionPool


class RawHTTPConnectionPool(HTTPConnectionPool):
    def __init__(self, *args, **kwargs):
        super(RawHTTPConnectionPool, self).__init__(*args, **kwargs)
        self.ConnectionCls = RawHTTPConnection


class RawHTTPSConnectionPool(HTTPSConnectionPool):
    def __init__(self, *args, **kwargs):
        super(RawHTTPSConnectionPool, self).__init__(*args, **kwargs)
        self.ConnectionCls = RawHTTPSConnection



#!/usr/bin/env python3

from os import path
from setuptools import setup, find_packages

__folder__ = path.abspath(path.dirname(__file__))

with open(path.join(__folder__, 'README.md')) as readme_file:
    long_description = readme_file.read()

about = {}
with open(path.join(__folder__, 'requests_raw', '__version__.py')) as about_file:
    exec(about_file.read(), about)

with open(path.join(__folder__, 'requirements.txt')) as req_file:
    install_requires = req_file.readlines()

setup(
    name=about['__title__'],
    version=about['__version__']+".post0",
    description=about['__description__'],
    long_description=long_description,
    long_description_content_type='text/markdown',
    author=about['__author__'],
    author_email=about['__author_email__'],
    packages=find_packages(exclude=['examples', 'tests']),
    python_requires=">=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*",
    install_requires=install_requires,
    license=about['__license__'],
    platforms='any',
    url='https://github.com/realgam3/requests-raw',
    zip_safe=False,
    classifiers=[
        'License :: OSI Approved :: Apache Software License',
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Science/Research',
        'Natural Language :: English',
        'Operating System :: POSIX :: Linux',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy'
    ],
    extras_require={
        'security': ['pyOpenSSL >= 0.14', 'cryptography>=1.3.4'],
        'socks': ['PySocks>=1.5.6, !=1.5.7'],
        'socks:sys_platform == "win32" and python_version == "2.7"': ['win_inet_pton'],
    },
    project_urls={
        'Source': 'https://github.com/realgam3/requests-raw',
    },
)



import requests
from .adapters import RawAdapter
from .__version__ import __title__


class Session(requests.Session):
    def __init__(self):
        super(Session, self).__init__()
        self.mount("http://", RawAdapter())
        self.mount("https://", RawAdapter())

    def raw(self, url, data, **kwargs):
        kwargs.setdefault('allow_redirects', True)
        return self.request(__title__, url, data=data, **kwargs)


def session():
    return Session()



from requests.adapters import HTTPAdapter
from .connectionpool import RawHTTPConnectionPool, RawHTTPSConnectionPool


class RawAdapter(HTTPAdapter):
    def __init__(self, *args, **kwargs):
        super(RawAdapter, self).__init__(*args, **kwargs)
        self.poolmanager.pool_classes_by_scheme["http"] = RawHTTPConnectionPool
        self.poolmanager.key_fn_by_scheme["http"] = self.poolmanager.key_fn_by_scheme["http"]

        self.poolmanager.pool_classes_by_scheme["https"] = RawHTTPSConnectionPool
        self.poolmanager.key_fn_by_scheme["https"] = self.poolmanager.key_fn_by_scheme["https"]
